package Image::WMF::Constants;
use strict;
use vars qw(    
            @ISA
			@EXPORT
            @EXPORT_OK
            %EXPORT_TAGS
			%FONTNAME
            $VERSION
			$PS_SOLID
			$PS_DASH
			$PS_DOT
			$PS_DASHDOT
			$PS_DASHDOTDOT
			$PS_NULL
			$PS_INSIDEFRAME
			$BS_SOLID
			$BS_HOLLOW
			$BS_NULL
			$BS_HATCHED
			$BS_PATTERN
			$BS_DIBPATTERN
			$HS_HORIZONTAL
			$HS_VERTICAL
			$HS_FDIAGONAL
			$HS_BDIAGONAL
			$HS_CROSS
			$HS_DIAGCROSS
			$DIB_RGB_COLORS
			$DIB_PAL_COLORS
			$FW_DONTCARE
			$FW_THIN
			$FW_NORMAL
			$FW_BOLD
			$FW_BLACK
			$ANSI_CHARSET
			$DEFAULT_CHARSET
			$SYMBOL_CHARSET
			$SHIFTJIS_CHARSET
			$OEM_CHARSET
			$OUT_DEFAULT_PRECIS
			$OUT_STRING_PRECIS
			$OUT_CHARACTER_PRECIS
			$OUT_STROKE_PRECIS
			$OUT_TT_PRECIS
			$OUT_DEVICE_PRECIS
			$OUT_RASTER_PRECIS
			$CLIP_DEFAULT_PRECIS
			$CLIP_CHARACTER_PRECIS
			$CLIP_STROKE_PRECIS
			$CLIP_MASK
			$CLIP_LH_ANGLES
			$CLIP_TT_ALWAYS
			$DEFAULT_QUALITY
			$DRAFT_QUALITY
			$PROOF_QUALITY
			$DEFAULT_PITCH
			$FIXED_PITCH
			$VARIABLE_PITCH
			$FF_DONTCARE
			$FF_ROMAN
			$FF_SWISS
			$FF_MODERN
			$FF_SCRIPT
			$FF_DECORATIVE
			$TRANSPARENT
			$OPAQUE
			$FLOODFILLBORDER
			$FLOODFILLSURFACE
			$MM_TEXT
			$MM_LOMETRIC
			$MM_HIMETRIC
			$MM_LOENGLISH
			$MM_HIENGLISH
			$MM_HITWIPS
			$MM_ISOTROPIC
			$MM_ANISOTROPIC
			$ALTERNATE
			$WINDING
			$STRETCH_ANDSCANS
			$STRETCH_ORSCANS
			$STRETCH_DELETESCANS
			$TA_TOP
			$TA_BOTTOM
			$TA_BASELINE
			$TA_LEFT
			$TA_RIGHT
			$TA_CENTER
			$TA_NOUPDATECP
			$TA_UPDATECP
			$R2_BLACK
			$R2_NOTMERGEPEN
			$R2_MASKNOTPENNOT
			$R2_NOTCOPYPEN
			$R2_MASKPENNOT
			$R2_NOT
			$R2_XORPEN
			$R2_NOTMASKPEN
			$R2_MASKPEN
			$R2_NOTXORPEN
			$R2_NOP
			$R2_MERGENOTPEN
			$R2_COPYPEN
			$R2_MERGEPENNOT
			$R2_MERGEPEN
			$R2_WHITE
			$ETO_OPAQUE
			$ETO_CLIPPED
			$BLACKNESS
			$NOTSRCERASE
			$NOTSRCCOPY
			$SRCERASE
			$DSTINVERT
			$PATINVERT
			$SRCINVERT
			$SRCAND
			$MERGEPAINT
			$SRCCOPY
			$SRCPAINT
			$PATCOPY
			$PATPAINT
			$WHITENESS
            );
use Exporter();
@ISA=qw(Exporter);
$VERSION=1.00;

%FONTNAME = (
	'helvetica' 	=> "Arial",
	'timesroman' 	=> "Times New Roman",
	'courier' 		=> "Courier New",
	'zapfdingsbat' 	=> "Windings",
);

$PS_SOLID = 0;
$PS_DASH = 1;
$PS_DOT = 2;
$PS_DASHDOT = 3;
$PS_DASHDOTDOT = 4;
$PS_NULL = 5;
$PS_INSIDEFRAME = 6;
$BS_SOLID = 0;
$BS_HOLLOW = 1;
$BS_NULL = 1;
$BS_HATCHED = 2;
$BS_PATTERN = 3;
$BS_DIBPATTERN = 5;
$HS_HORIZONTAL = 0;
$HS_VERTICAL = 1;
$HS_FDIAGONAL = 2;
$HS_BDIAGONAL = 3;
$HS_CROSS = 4;
$HS_DIAGCROSS = 5;
$DIB_RGB_COLORS = 0;
$DIB_PAL_COLORS = 1;
$FW_DONTCARE = 100;
$FW_THIN = 100;
$FW_NORMAL = 400;
$FW_BOLD = 700;
$FW_BLACK = 900;
$ANSI_CHARSET = 0;
$DEFAULT_CHARSET = 1;
$SYMBOL_CHARSET = 2;
$SHIFTJIS_CHARSET = -128;
$OEM_CHARSET = -1;
$OUT_DEFAULT_PRECIS = 0;
$OUT_STRING_PRECIS = 1;
$OUT_CHARACTER_PRECIS = 2;
$OUT_STROKE_PRECIS = 3;
$OUT_TT_PRECIS = 4;
$OUT_DEVICE_PRECIS = 5;
$OUT_RASTER_PRECIS = 6;
$CLIP_DEFAULT_PRECIS = 0;
$CLIP_CHARACTER_PRECIS = 1;
$CLIP_STROKE_PRECIS = 2;
$CLIP_MASK = 15;
$CLIP_LH_ANGLES = 16;
$CLIP_TT_ALWAYS = 32;
$DEFAULT_QUALITY = 0;
$DRAFT_QUALITY = 1;
$PROOF_QUALITY = 2;
$DEFAULT_PITCH = 0;
$FIXED_PITCH = 1;
$VARIABLE_PITCH = 2;
$FF_DONTCARE = 0;
$FF_ROMAN = 16;
$FF_SWISS = 32;
$FF_MODERN = 48;
$FF_SCRIPT = 64;
$FF_DECORATIVE = 80;
$TRANSPARENT = 1;
$OPAQUE = 2;
$FLOODFILLBORDER = 0;
$FLOODFILLSURFACE = 1;
$MM_TEXT = 1;
$MM_LOMETRIC = 2;
$MM_HIMETRIC = 3;
$MM_LOENGLISH = 4;
$MM_HIENGLISH = 5;
$MM_HITWIPS = 6;
$MM_ISOTROPIC = 7;
$MM_ANISOTROPIC = 8;
$ALTERNATE = 1;
$WINDING = 2;
$STRETCH_ANDSCANS = 1;
$STRETCH_ORSCANS = 2;
$STRETCH_DELETESCANS = 3;
$TA_TOP = 0;
$TA_BOTTOM = 8;
$TA_BASELINE = 24;
$TA_LEFT = 0;
$TA_RIGHT = 2;
$TA_CENTER = 6;
$TA_NOUPDATECP = 0;
$TA_UPDATECP = 1;
$R2_BLACK = 1;
$R2_NOTMERGEPEN = 2;
$R2_MASKNOTPENNOT = 3;
$R2_NOTCOPYPEN = 4;
$R2_MASKPENNOT = 5;
$R2_NOT = 6;
$R2_XORPEN = 7;
$R2_NOTMASKPEN = 8;
$R2_MASKPEN = 9;
$R2_NOTXORPEN = 10;
$R2_NOP = 11;
$R2_MERGENOTPEN = 12;
$R2_COPYPEN = 13;
$R2_MERGEPENNOT = 14;
$R2_MERGEPEN = 15;
$R2_WHITE = 16;
$ETO_OPAQUE = 2;
$ETO_CLIPPED = 4;
$BLACKNESS = 66;
$NOTSRCERASE = 0x1100a6;
$NOTSRCCOPY = 0x330008;
$SRCERASE = 0x440328;
$DSTINVERT = 0x550009;
$PATINVERT = 0x5a0049;
$SRCINVERT = 0x660046;
$SRCAND = 0x8800c6;
$MERGEPAINT = 0xbb0226;
$SRCCOPY = 0xcc0020;
$SRCPAINT = 0xee0086;
$PATCOPY = 0xf00021;
$PATPAINT = 0xfb0a09;
$WHITENESS = 0xff0062;

# The default exported symbols (all constants)

@EXPORT = qw(   
			%FONTNAME
			$PS_SOLID
			$PS_DASH
			$PS_DOT
			$PS_DASHDOT
			$PS_DASHDOTDOT
			$PS_NULL
			$PS_INSIDEFRAME
			$BS_SOLID
			$BS_HOLLOW
			$BS_NULL
			$BS_HATCHED
			$BS_PATTERN
			$BS_DIBPATTERN
			$HS_HORIZONTAL
			$HS_VERTICAL
			$HS_FDIAGONAL
			$HS_BDIAGONAL
			$HS_CROSS
			$HS_DIAGCROSS
			$DIB_RGB_COLORS
			$DIB_PAL_COLORS
			$FW_DONTCARE
			$FW_THIN
			$FW_NORMAL
			$FW_BOLD
			$FW_BLACK
			$ANSI_CHARSET
			$DEFAULT_CHARSET
			$SYMBOL_CHARSET
			$SHIFTJIS_CHARSET
			$OEM_CHARSET
			$OUT_DEFAULT_PRECIS
			$OUT_STRING_PRECIS
			$OUT_CHARACTER_PRECIS
			$OUT_STROKE_PRECIS
			$OUT_TT_PRECIS
			$OUT_DEVICE_PRECIS
			$OUT_RASTER_PRECIS
			$CLIP_DEFAULT_PRECIS
			$CLIP_CHARACTER_PRECIS
			$CLIP_STROKE_PRECIS
			$CLIP_MASK
			$CLIP_LH_ANGLES
			$CLIP_TT_ALWAYS
			$DEFAULT_QUALITY
			$DRAFT_QUALITY
			$PROOF_QUALITY
			$DEFAULT_PITCH
			$FIXED_PITCH
			$VARIABLE_PITCH
			$FF_DONTCARE
			$FF_ROMAN
			$FF_SWISS
			$FF_MODERN
			$FF_SCRIPT
			$FF_DECORATIVE
			$TRANSPARENT
			$OPAQUE
			$FLOODFILLBORDER
			$FLOODFILLSURFACE
			$MM_TEXT
			$MM_LOMETRIC
			$MM_HIMETRIC
			$MM_LOENGLISH
			$MM_HIENGLISH
			$MM_HITWIPS
			$MM_ISOTROPIC
			$MM_ANISOTROPIC
			$ALTERNATE
			$WINDING
			$STRETCH_ANDSCANS
			$STRETCH_ORSCANS
			$STRETCH_DELETESCANS
			$TA_TOP
			$TA_BOTTOM
			$TA_BASELINE
			$TA_LEFT
			$TA_RIGHT
			$TA_CENTER
			$TA_NOUPDATECP
			$TA_UPDATECP
			$R2_BLACK
			$R2_NOTMERGEPEN
			$R2_MASKNOTPENNOT
			$R2_NOTCOPYPEN
			$R2_MASKPENNOT
			$R2_NOT
			$R2_XORPEN
			$R2_NOTMASKPEN
			$R2_MASKPEN
			$R2_NOTXORPEN
			$R2_NOP
			$R2_MERGENOTPEN
			$R2_COPYPEN
			$R2_MERGEPENNOT
			$R2_MERGEPEN
			$R2_WHITE
			$ETO_OPAQUE
			$ETO_CLIPPED
			$BLACKNESS
			$NOTSRCERASE
			$NOTSRCCOPY
			$SRCERASE
			$DSTINVERT
			$PATINVERT
			$SRCINVERT
			$SRCAND
			$MERGEPAINT
			$SRCCOPY
			$SRCPAINT
			$PATCOPY
			$PATPAINT
			$WHITENESS
			);

1;
